# coding: utf-8

"""
    Climify API

    Provides a REST endpoints for Climify applications and 3rd party solutions.  # noqa: E501

    The version of the OpenAPI document: 1.2.3
    Generated by: https://openapi-generator.tech
"""

import copy
import logging
import multiprocessing
import os
import sys
import urllib3
from varname import nameof
import typing

from http import client as http_client
from enum import Enum

import yaml
from climify_api.exceptions import ApiValueError, MissingConfigurationError
from climify_api.auth import AuthenticationMethod


class ClimifyRegion(Enum):
    DEV = "dev"
    EU = "eu"
    CN = "cn"
    US = "us"

JSON_SCHEMA_VALIDATION_KEYWORDS = {
    'multipleOf', 'maximum', 'exclusiveMaximum',
    'minimum', 'exclusiveMinimum', 'maxLength',
    'minLength', 'pattern', 'maxItems', 'minItems',
    'uniqueItems', 'maxProperties', 'minProperties',
}

DEFAULT_CONFIG_PATH = 'config.yml'

def _read_configuration_file(config_path: str) -> dict:
    """
    Reads a yml configuration file and returns the resulting dictionary.

    Arguments:
        - config_path: String containing the relative or absolute path to the configuration file.
    """
    with open(config_path, "r") as f:
        return yaml.safe_load(f)

def _get_item_from_configuration(configuration: dict, key: str):
    """Returns the value of a configuration item with the given key
    or aliases of the given key

    Args:
        configuration (dict): Configuration dictionary
        key (str): Key to extract

    Raises:

    Returns:
        object: Value associated with the key if it exists
    """
    configuration_keys = configuration.keys()
    if key.upper() in configuration_keys:
        return configuration.get(key.upper())
    
    if key.lower() in configuration_keys:
        return configuration.get(key.lower())
    
    if key.capitalize() in configuration_keys:
        return configuration.get(key.capitalize())
    
    return None

class Configuration(object):
    """NOTE: This class is auto generated by OpenAPI Generator and modified by Peter Bonvang.
    Either specify the configurations in a yml file and parse the file path to the constructor
    or provide the configurations directly in the constructor.

    Ref: https://openapi-generator.tech

    :param configuration_file_path: Path to configuration file. Will use 'config.yml' if none is specified.
    :param region: Climify region to connect to
    :param authentication_method: Sets the authentication method. Default: User credentials
    :param oauth_client_id: OAuth client id to be used to obtain access tokens
    :param string disabled_client_side_validations: Comma-separated list of
      JSON schema validation keywords to disable JSON schema structural validation
      rules. The following keywords may be specified: multipleOf, maximum,
      exclusiveMaximum, minimum, exclusiveMinimum, maxLength, minLength, pattern,
      maxItems, minItems.
      By default, the validation is performed for data generated locally by the client
      and data received from the server, independent of any validation performed by
      the server side. If the input data does not satisfy the JSON schema validation
      rules specified in the OpenAPI document, an exception is raised.
      If disabled_client_side_validations is set, structural validation is
      disabled. This can be useful to troubleshoot data validation problem, such as
      when the OpenAPI document validation rules do not match the actual API data
      received by the server.
    :param server_variables: Mapping with string values to replace variables in
      templated server configuration. The validation of enums is performed for
      variables with defined enum values before.
    :param server_operation_index: Mapping from operation ID to an index to server
      configuration.
    :param server_operation_variables: Mapping from operation ID to a mapping with
      string values to replace variables in templated server configuration.
      The validation of enums is performed for variables with defined enum values before.

    """

    _default = None

    def __init__(
        self,
        configuration_file_path: str = None,
        region: typing.Union[str, ClimifyRegion] = None,
        authentication_method: AuthenticationMethod = AuthenticationMethod.USER_CREDENTIALS,
        oauth_client_id: str = None,
        disabled_client_side_validations: str = "",
        server_variables=None,
        server_operation_index=None,
        server_operation_variables=None,
    ):
        """Constructor
        """
        
        if os.path.exists(DEFAULT_CONFIG_PATH):
            configuration_file_path = DEFAULT_CONFIG_PATH
            
        if configuration_file_path is not None:
            config = _read_configuration_file(configuration_file_path)
            region = region or _get_item_from_configuration(config, nameof(region))
            authentication_method = authentication_method or _get_item_from_configuration(config, nameof(authentication_method))
            oauth_client_id = oauth_client_id or _get_item_from_configuration(config, nameof(oauth_client_id))
            server_variables = server_variables or _get_item_from_configuration(config, nameof(server_variables))
            server_operation_index = server_operation_index or _get_item_from_configuration(config, nameof(server_operation_index))
            server_operation_variables = server_operation_variables or _get_item_from_configuration(config, nameof(server_operation_variables))
        
        self.authentication_method = authentication_method
        
        if authentication_method == AuthenticationMethod.USER_CREDENTIALS and oauth_client_id is None:
            raise MissingConfigurationError(nameof(oauth_client_id))
        self.oauth_client_id = oauth_client_id

        if region is None:
            raise MissingConfigurationError(nameof(region))
        region = region.value if isinstance(region, ClimifyRegion) else region
        self.region = region.lower()

        self.host = f"https://app.{self.region}.climify.com/api"
        self.token_url = f"https://auth.{self.region}.climify.com/realms/climify/protocol/openid-connect/token"

        self.server_operation_index = server_operation_index or {}
        """Default server index
        """
        self.server_variables = server_variables or {}
        self.server_operation_variables = server_operation_variables or {}
        """Default server variables
        """
        self.temp_folder_path = None
        """Temp file folder for downloading files
        """
        # Authentication Settings
        self.disabled_client_side_validations = disabled_client_side_validations
        self.logger = {}
        """Logging Settings
        """
        self.logger["package_logger"] = logging.getLogger("openapi_client")
        self.logger["urllib3_logger"] = logging.getLogger("urllib3")
        self.logger_format = '%(asctime)s %(levelname)s %(message)s'
        """Log format
        """
        self.logger_stream_handler = None
        """Log stream handler
        """
        self.logger_file_handler = None
        """Log file handler
        """
        self.logger_file = None
        """Debug file location
        """
        self.debug = False
        """Debug switch
        """

        self.verify_ssl = True
        """SSL/TLS verification
           Set this to false to skip verifying SSL certificate when calling API
           from https server.
        """
        self.ssl_ca_cert = None
        """Set this to customize the certificate file to verify the peer.
        """
        self.cert_file = None
        """client certificate file
        """
        self.key_file = None
        """client key file
        """
        self.assert_hostname = None
        """Set this to True/False to enable/disable SSL hostname verification.
        """

        self.connection_pool_maxsize = multiprocessing.cpu_count() * 5
        """urllib3 connection pool's maximum number of connections saved
           per pool. urllib3 uses 1 connection as default value, but this is
           not the best value when you are making a lot of possibly parallel
           requests to the same host, which is often the case here.
           cpu_count * 5 is used as default value to increase performance.
        """

        self.proxy = None
        """Proxy URL
        """
        self.proxy_headers = None
        """Proxy headers
        """
        self.safe_chars_for_path_param = ''
        """Safe chars for path_param
        """
        self.retries = None
        """Adding retries to override urllib3 default value 3
        """
        # Enable client side validation
        self.client_side_validation = True

        # Options to pass down to the underlying urllib3 socket
        self.socket_options = None

    def __deepcopy__(self, memo):
        cls = self.__class__
        result = cls.__new__(cls)
        memo[id(self)] = result
        for k, v in self.__dict__.items():
            if k not in ('logger', 'logger_file_handler'):
                setattr(result, k, copy.deepcopy(v, memo))
        # shallow copy of loggers
        result.logger = copy.copy(self.logger)
        # use setters to configure loggers
        result.logger_file = self.logger_file
        result.debug = self.debug
        return result

    def __setattr__(self, name, value):
        object.__setattr__(self, name, value)
        if name == 'disabled_client_side_validations':
            s = set(filter(None, value.split(',')))
            for v in s:
                if v not in JSON_SCHEMA_VALIDATION_KEYWORDS:
                    raise ApiValueError(
                        "Invalid keyword: '{0}''".format(v))
            self._disabled_client_side_validations = s

    @classmethod
    def set_default(cls, default):
        """Set default instance of configuration.

        It stores default configuration, which can be
        returned by get_default_copy method.

        :param default: object of Configuration
        """
        cls._default = copy.deepcopy(default)

    @classmethod
    def get_default_copy(cls):
        """Return new instance of configuration.

        This method returns newly created, based on default constructor,
        object of Configuration class or returns a copy of default
        configuration passed by the set_default method.

        :return: The configuration object.
        """
        if cls._default is not None:
            return copy.deepcopy(cls._default)
        return Configuration()

    @property
    def logger_file(self):
        """The logger file.

        If the logger_file is None, then add stream handler and remove file
        handler. Otherwise, add file handler and remove stream handler.

        :param value: The logger_file path.
        :type: str
        """
        return self.__logger_file

    @logger_file.setter
    def logger_file(self, value):
        """The logger file.

        If the logger_file is None, then add stream handler and remove file
        handler. Otherwise, add file handler and remove stream handler.

        :param value: The logger_file path.
        :type: str
        """
        self.__logger_file = value
        if self.__logger_file:
            # If set logging file,
            # then add file handler and remove stream handler.
            self.logger_file_handler = logging.FileHandler(self.__logger_file)
            self.logger_file_handler.setFormatter(self.logger_formatter)
            for _, logger in self.logger.items():
                logger.addHandler(self.logger_file_handler)

    @property
    def debug(self):
        """Debug status

        :param value: The debug status, True or False.
        :type: bool
        """
        return self.__debug

    @debug.setter
    def debug(self, value):
        """Debug status

        :param value: The debug status, True or False.
        :type: bool
        """
        self.__debug = value
        if self.__debug:
            # if debug status is True, turn on debug logging
            for _, logger in self.logger.items():
                logger.setLevel(logging.DEBUG)
            # turn on http_client debug
            http_client.HTTPConnection.debuglevel = 1
        else:
            # if debug status is False, turn off debug logging,
            # setting log level to default `logging.WARNING`
            for _, logger in self.logger.items():
                logger.setLevel(logging.WARNING)
            # turn off http_client debug
            http_client.HTTPConnection.debuglevel = 0

    @property
    def logger_format(self):
        """The logger format.

        The logger_formatter will be updated when sets logger_format.

        :param value: The format string.
        :type: str
        """
        return self.__logger_format

    @logger_format.setter
    def logger_format(self, value):
        """The logger format.

        The logger_formatter will be updated when sets logger_format.

        :param value: The format string.
        :type: str
        """
        self.__logger_format = value
        self.logger_formatter = logging.Formatter(self.__logger_format)

    def get_api_key_with_prefix(self, identifier, alias=None):
        """Gets API key (with prefix if set).

        :param identifier: The identifier of apiKey.
        :param alias: The alternative identifier of apiKey.
        :return: The token for api key authentication.
        """
        if self.refresh_api_key_hook is not None:
            self.refresh_api_key_hook(self)
        key = self.api_key.get(identifier, self.api_key.get(
            alias) if alias is not None else None)
        if key:
            prefix = self.api_key_prefix.get(identifier)
            if prefix:
                return "%s %s" % (prefix, key)
            else:
                return key

    def get_basic_auth_token(self):
        """Gets HTTP basic authentication header (string).

        :return: The token for basic HTTP authentication.
        """
        username = ""
        if self.username is not None:
            username = self.username
        password = ""
        if self.password is not None:
            password = self.password
        return urllib3.util.make_headers(
            basic_auth=username + ':' + password
        ).get('authorization')

    def auth_settings(self):
        """Gets Auth Settings dict for api client.

        :return: The Auth Settings information dict.
        """
        auth = {}
        return auth

    def to_debug_report(self):
        """Gets the essential information for debugging.

        :return: The report for debugging.
        """
        return "Python SDK Debug Report:\n"\
               "OS: {env}\n"\
               "Python Version: {pyversion}\n"\
               "Version of the API: 1.2.3\n"\
               "SDK Package Version: 1.0.0".\
               format(env=sys.platform, pyversion=sys.version)

    def get_host_settings(self):
        """Gets an array of host settings

        :return: An array of host settings
        """
        return {
            'dev': {
                'url': 'https://app.dev.climify.com/api',
                'description': 'Development environment'
            },
            'eu': {
                'url': 'https://app.eu.climify.com/api',
                'description': 'Production environment, EU region'
            },
            'us': {
                'url': 'https://app.us.climify.com/api',
                'description': 'Production environment, US region'
            },
            'cn': {
                'url': 'https://app.cn.climify.com/api',
                'description': 'Production environment, CN region'
            }
        }

    def get_host_from_settings(self, region, variables=None, servers=None):
        """Gets host URL based on the index and variables
        :param region: region to connect to
        :param variables: hash of variable and the corresponding value
        :param servers: an array of host settings or None
        :return: URL based on host settings
        """
        if region is None:
            return self._base_path

        variables = {} if variables is None else variables
        servers = self.get_host_settings() if servers is None else servers

        try:
            server = servers[region]
        except IndexError:
            raise ValueError(
                "Invalid index {0} when selecting the host settings. "
                "Must be less than {1}".format(region, len(servers)))

        url = server['url']

        # go through variables and replace placeholders
        for variable_name, variable in server.get('variables', {}).items():
            used_value = variables.get(
                variable_name, variable['default_value'])

            if 'enum_values' in variable \
                    and used_value not in variable['enum_values']:
                raise ValueError(
                    "The variable `{0}` in the host URL has invalid value "
                    "{1}. Must be {2}.".format(
                        variable_name, variables[variable_name],
                        variable['enum_values']))

            url = url.replace("{" + variable_name + "}", used_value)

        return url

    @property
    def host(self):
        """Return generated host."""
        return self.get_host_from_settings(self.region, variables=self.server_variables)

    @host.setter
    def host(self, value):
        """Fix base path."""
        self._base_path = value
        self.server_index = None
